@@ -75,6 +75,8 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <time.h>
+#include <sys/time.h>
 #include <sys/stat.h>
 
 /* OpenSSL headers */
@@ -104,6 +106,7 @@
 /* mod_ssl headers */
 #include "ssl_expr.h"
 #include "ssl_util_ssl.h"
+#include "ssl_util_table.h"
 
 /*
  * Provide reasonable default for some defines
@@ -256,6 +259,11 @@
 #include <process.h>
 #endif
 
+/*
+ * Support for MM library
+ */
+#define SSL_MM_FILE_MODE ( S_IRUSR|S_IWUSR )
+
 /*
  * Support for DBM library
  */
@@ -466,6 +474,9 @@ typedef struct {
     DH             *pDHTmpParam1024;
     int             nSessionCacheMode;
     char           *szSessionCacheDataFile;
+    int             nSessionCacheDataSize;
+    AP_MM          *pSessionCacheDataMM;
+    table_t        *tSessionCacheDataTable;
     ssl_mutexmode_t nMutexMode;
     char           *szMutexFile;
     int             nMutexFD;
@@ -612,13 +623,18 @@ void         ssl_scache_init(server_rec *, pool *);
 void         ssl_scache_store(server_rec *, SSL_SESSION *, int);
 SSL_SESSION *ssl_scache_retrieve(server_rec *, UCHAR *, int);
 void         ssl_scache_remove(server_rec *, SSL_SESSION *);
-void         ssl_scache_expire(server_rec *);
+void         ssl_scache_expire(server_rec *, time_t);
 char        *ssl_scache_id2sz(UCHAR *, int);
 void         ssl_scache_dbm_init(server_rec *, pool *);
 void         ssl_scache_dbm_store(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_retrieve(server_rec *, ssl_scinfo_t *);
 void         ssl_scache_dbm_remove(server_rec *, ssl_scinfo_t *);
-void         ssl_scache_dbm_expire(server_rec *);
+void         ssl_scache_dbm_expire(server_rec *, time_t);
+void         ssl_scache_shm_init(server_rec *, pool *);
+void         ssl_scache_shm_store(server_rec *, ssl_scinfo_t *);
+void         ssl_scache_shm_retrieve(server_rec *, ssl_scinfo_t *);
+void         ssl_scache_shm_remove(server_rec *, ssl_scinfo_t *);
+void         ssl_scache_shm_expire(server_rec *, time_t);
 
 /*  Pass Phrase Support  */
 void         ssl_pphrase_Handle(server_rec *, pool *);
