@@ -1,8 +1,8 @@
 /*                      _             _
-**  _ __ ___   ___   __| |    ___ ___| |
-** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
-** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
+**  _ __ ___   ___   __| |    ___ ___| |  mod_ssl
+** | '_ ` _ \ / _ \ / _` |   / __/ __| |  Apache Interface to OpenSSL
+** | | | | | | (_) | (_| |   \__ \__ \ |  www.modssl.org
+** |_| |_| |_|\___/ \__,_|___|___/___/_|  ftp.modssl.org
 **                      |_____|
 **  ssl_engine_pphrase.c
 **  Pass Phrase Dialog
@@ -27,7 +27,7 @@
  *    software must display the following acknowledgment:
  *    "This product includes software developed by
  *     Ralf S. Engelschall <rse@engelschall.com> for use in the
- *     mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)."
+ *     mod_ssl project (http://www.modssl.org/)."
  *
  * 4. The names "mod_ssl" must not be used to endorse or promote
  *    products derived from this software without prior written
@@ -42,7 +42,7 @@
  *    acknowledgment:
  *    "This product includes software developed by
  *     Ralf S. Engelschall <rse@engelschall.com> for use in the
- *     mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)."
+ *     mod_ssl project (http://www.modssl.org/)."
  *
  * THIS SOFTWARE IS PROVIDED BY RALF S. ENGELSCHALL ``AS IS'' AND ANY
  * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -83,9 +83,9 @@ void ssl_pphrase_Handle(server_rec *s, pool *p)
     server_rec *pServ;
     char *cpVHostID;
     char szPath[MAX_STRING_LEN];
+    EVP_PKEY *pPrivateKey;
     ssl_asn1_t *asn1;
     unsigned char *ucp;
-    RSA *pRSAKey;
     X509 *pX509Cert;
     FILE *fp;
     BOOL bReadable;
@@ -98,6 +98,10 @@ void ssl_pphrase_Handle(server_rec *s, pool *p)
     int nPassPhraseDialogCur;
     BOOL bPassPhraseDialogOnce;
     char **cpp;
+    int i, j;
+    ssl_algo_t algoCert, algoKey, at;
+    char *an;
+    char *cp;
 
     /*
      * Start with a fresh pass phrase array
@@ -116,194 +120,245 @@ void ssl_pphrase_Handle(server_rec *s, pool *p)
             continue;
 
         cpVHostID = ssl_util_vhostid(p, pServ);
-        ssl_log(pServ, SSL_LOG_INFO, 
-                "Init: Loading certificate & private key of SSL-aware server %s", 
+        ssl_log(pServ, SSL_LOG_INFO,
+                "Init: Loading certificate & private key of SSL-aware server %s",
                 cpVHostID);
 
         /*
-         * Read in server certificate: This is the easy part
+         * Read in server certificate(s): This is the easy part
          * because this file isn't encrypted in any way.
          */
-        if (sc->szCertificateFile == NULL) {
+        if (sc->szPublicCertFile[0] == NULL) {
             ssl_log(pServ, SSL_LOG_ERROR,
                     "Init: Server %s should be SSL-aware but has no certificate configured "
                     "[Hint: SSLCertifcateFile]", cpVHostID);
             ssl_die();
         }
-        ap_cpystrn(szPath, sc->szCertificateFile, sizeof(szPath));
-        if ((fp = ap_pfopen(p, szPath, "r")) == NULL) {
-            ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
-                    "Init: Can't open server certificate file %s", szPath);
-            ssl_die();
-        }
-        if ((pX509Cert = SSL_read_X509(fp, NULL, NULL)) == NULL) {
-            ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
-                    "Init: Unable to read server certificate from file %s", szPath);
-            ssl_die();
-        }
-        ap_pfclose(p, fp);
+        algoCert = SSL_ALGO_UNKNOWN;
+        algoKey  = SSL_ALGO_UNKNOWN;
+        for (i = 0, j = 0; i < SSL_AIDX_MAX && sc->szPublicCertFile[i] != NULL; i++) {
 
-        /*
-         * Insert the certificate into global module configuration to let it
-         * survive the processing between the 1st Apache API init round (where
-         * we operate here) and the 2nd Apache init round (where the
-         * certificate is actually used to configure mod_ssl's per-server
-         * configuration structures).
-         */
-        asn1 = (ssl_asn1_t *)ssl_ds_table_push(mc->tPublicCert, cpVHostID);
-        asn1->nData  = i2d_X509(pX509Cert, NULL);
-        asn1->cpData = ap_palloc(mc->pPool, asn1->nData);
-        ucp = asn1->cpData; i2d_X509(pX509Cert, &ucp); /* 2nd arg increments */
+            ap_cpystrn(szPath, sc->szPublicCertFile[i], sizeof(szPath));
+            if ((fp = ap_pfopen(p, szPath, "r")) == NULL) {
+                ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
+                        "Init: Can't open server certificate file %s", szPath);
+                ssl_die();
+            }
+            if ((pX509Cert = SSL_read_X509(fp, NULL, NULL)) == NULL) {
+                ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                        "Init: Unable to read server certificate from file %s", szPath);
+                ssl_die();
+            }
+            ap_pfclose(p, fp);
 
-        /*
-         * Free the X509 structure
-         */
-        X509_free(pX509Cert);
+            /*
+             * check algorithm type of certificate and make
+             * sure only one certificate per type is used.
+             */
+            at = ssl_util_algotypeof(pX509Cert, NULL);
+            an = ssl_util_algotypestr(at);
+            if (algoCert & at) {
+                ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                        "Init: Multiple %s server certificates not allowed", an);
+                ssl_die();
+            }
+            algoCert |= at;
 
-        /*
-         * Read in the private key: This is the non-trivial part, because the
-         * key is typically encrypted, so a pass phrase dialog has to be used
-         * to request it from the user (or it has to be alternatively gathered
-         * from a dialog program). The important point here is that ISPs
-         * usually have hundrets of virtual servers configured and a lot of
-         * them use SSL, so really we have to minimize the pass phrase
-         * dialogs.
-         *
-         * The idea is this: When N virtual hosts are configured and all of
-         * them use encrypted private keys with different pass phrases, we
-         * have no chance and have to pop up N pass phrase dialogs. But
-         * usually the admin is clever enough and uses the same pass phrase
-         * for more private key files (typically he even uses one single pass
-         * phrase for all). When this is the case we can minimize the dialogs
-         * by trying to re-use already known/entered pass phrases.
-         */
-        if (sc->szKeyFile)
-            ap_cpystrn(szPath, sc->szKeyFile, sizeof(szPath));
+            /*
+             * Insert the certificate into global module configuration to let it
+             * survive the processing between the 1st Apache API init round (where
+             * we operate here) and the 2nd Apache init round (where the
+             * certificate is actually used to configure mod_ssl's per-server
+             * configuration structures).
+             */
+            cp = ap_psprintf(mc->pPool, "%s:%s", cpVHostID, an);
+            asn1 = (ssl_asn1_t *)ssl_ds_table_push(mc->tPublicCert, cp);
+            asn1->nData  = i2d_X509(pX509Cert, NULL);
+            asn1->cpData = ap_palloc(mc->pPool, asn1->nData);
+            ucp = asn1->cpData; i2d_X509(pX509Cert, &ucp); /* 2nd arg increments */
 
-        /*
-         * Spread context variables for callback function
-         */
-        myCtxVarSet(mc, 1, pServ);
-        myCtxVarSet(mc, 2, p);
-        myCtxVarSet(mc, 3, aPassPhrase);
-        myCtxVarSet(mc, 4, &nPassPhraseCur);
-        myCtxVarSet(mc, 5, &cpPassPhraseCur);
-        myCtxVarSet(mc, 6, cpVHostID);
-        myCtxVarSet(mc, 7, &nPassPhraseDialog);
-        myCtxVarSet(mc, 8, &nPassPhraseDialogCur);
-        myCtxVarSet(mc, 9, &bPassPhraseDialogOnce);
+            /*
+             * Free the X509 structure
+             */
+            X509_free(pX509Cert);
 
-        nPassPhraseCur        = 0;
-        nPassPhraseRetry      = 0;
-        nPassPhraseDialogCur  = 0;
-        bPassPhraseDialogOnce = TRUE;
+            /*
+             * Read in the private key: This is the non-trivial part, because the
+             * key is typically encrypted, so a pass phrase dialog has to be used
+             * to request it from the user (or it has to be alternatively gathered
+             * from a dialog program). The important point here is that ISPs
+             * usually have hundrets of virtual servers configured and a lot of
+             * them use SSL, so really we have to minimize the pass phrase
+             * dialogs.
+             *
+             * The idea is this: When N virtual hosts are configured and all of
+             * them use encrypted private keys with different pass phrases, we
+             * have no chance and have to pop up N pass phrase dialogs. But
+             * usually the admin is clever enough and uses the same pass phrase
+             * for more private key files (typically he even uses one single pass
+             * phrase for all). When this is the case we can minimize the dialogs
+             * by trying to re-use already known/entered pass phrases.
+             */
+            if (sc->szPrivateKeyFile[j] != NULL)
+                ap_cpystrn(szPath, sc->szPrivateKeyFile[j++], sizeof(szPath));
 
-        for (;;) {
             /*
              * Try to read the private key file with the help of
              * the callback function which serves the pass
              * phrases to OpenSSL
              */
-            if ((fp = ap_pfopen(p, szPath, "r")) == NULL) {
-                ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
-                        "Init: Can't open server private key file %s", szPath);
+            myCtxVarSet(mc, 1, pServ);
+            myCtxVarSet(mc, 2, p);
+            myCtxVarSet(mc, 3, aPassPhrase);
+            myCtxVarSet(mc, 4, &nPassPhraseCur);
+            myCtxVarSet(mc, 5, &cpPassPhraseCur);
+            myCtxVarSet(mc, 6, cpVHostID);
+            myCtxVarSet(mc, 7, &nPassPhraseDialog);
+            myCtxVarSet(mc, 8, &nPassPhraseDialogCur);
+            myCtxVarSet(mc, 9, &bPassPhraseDialogOnce);
+
+            nPassPhraseCur        = 0;
+            nPassPhraseRetry      = 0;
+            nPassPhraseDialogCur  = 0;
+            bPassPhraseDialogOnce = TRUE;
+
+			pPrivateKey = NULL;
+
+            for (;;) {
+                /*
+                 * Try to read the private key file with the help of
+                 * the callback function which serves the pass
+                 * phrases to OpenSSL
+                 */
+                if ((fp = ap_pfopen(p, szPath, "r")) == NULL) {
+                    ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
+                            "Init: Can't open server private key file %s", szPath);
+                    ssl_die();
+                }
+                cpPassPhraseCur = NULL;
+                bReadable = ((pPrivateKey = SSL_read_PrivateKey(fp, NULL,
+                             ssl_pphrase_Handle_CB)) != NULL ? TRUE : FALSE);
+                ap_pfclose(p, fp);
+
+                /*
+                 * when the private key file now was readable,
+                 * it's fine and we go out of the loop
+                 */
+                if (bReadable)
+                   break;
+
+                /*
+                 * when we have more remembered pass phrases
+                 * try to reuse these first.
+                 */
+                if (nPassPhraseCur < nPassPhrase) {
+                    nPassPhraseCur++;
+                    continue;
+                }
+
+                /*
+                 * else it's not readable and we have no more
+                 * remembered pass phrases. Then this has to mean
+                 * that the callback function popped up the dialog
+                 * but a wrong pass phrase was entered.  We give the
+                 * user (but not the dialog program) a few more
+                 * chances...
+                 */
+                if (   sc->nPassPhraseDialogType == SSL_PPTYPE_BUILTIN
+                    && cpPassPhraseCur != NULL
+                    && nPassPhraseRetry < BUILTIN_DIALOG_RETRIES ) {
+                    fprintf(stdout, "Apache:mod_ssl:Error: Pass phrase incorrect "
+                            "(%d more retr%s permitted).\n",
+                            (BUILTIN_DIALOG_RETRIES-nPassPhraseRetry),
+                            (BUILTIN_DIALOG_RETRIES-nPassPhraseRetry) == 1 ? "y" : "ies");
+                    nPassPhraseRetry++;
+                    if (nPassPhraseRetry > BUILTIN_DIALOG_BACKOFF)
+                        sleep((nPassPhraseRetry-BUILTIN_DIALOG_BACKOFF)*5);
+                    continue;
+                }
+
+                /*
+                 * Ok, anything else now means a fatal error.
+                 */
+                if (cpPassPhraseCur == NULL)
+                    ssl_log(pServ, SSL_LOG_ERROR|SSL_ADD_SSLERR, "Init: Private key not found.");
+                    if (sc->nPassPhraseDialogType == SSL_PPTYPE_BUILTIN) {
+                        fprintf(stdout, "Apache:mod_ssl:Error: Private key not found.\n");
+                        fprintf(stdout, "**Stopped\n");
+                    }
+                else {
+                    ssl_log(pServ, SSL_LOG_ERROR|SSL_ADD_SSLERR, "Init: Pass phrase incorrect.");
+                    if (sc->nPassPhraseDialogType == SSL_PPTYPE_BUILTIN) {
+                        fprintf(stdout, "Apache:mod_ssl:Error: Pass phrase incorrect.\n");
+                        fprintf(stdout, "**Stopped\n");
+                    }
+                }
                 ssl_die();
             }
-            cpPassPhraseCur = NULL;
-            bReadable = ((pRSAKey = SSL_read_RSAPrivateKey(fp, NULL,
-                         ssl_pphrase_Handle_CB)) != NULL ? TRUE : FALSE);
-            ap_pfclose(p, fp);
 
-            /*
-             * when the private key file now was readable,
-             * it's fine and we go out of the loop
-             */
-            if (bReadable)
-               break;
-
-            /*
-             * when we have more remembered pass phrases
-             * try to reuse these first.
-             */
-            if (nPassPhraseCur < nPassPhrase) {
-                nPassPhraseCur++;
-                continue;
+            if (pPrivateKey == NULL) {
+                ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                        "Init: Unable to read server private key from file %s", szPath);
+                ssl_die();
             }
 
             /*
-             * else it's not readable and we have no more
-             * remembered pass phrases. Then this has to mean
-             * that the callback function popped up the dialog
-             * but a wrong pass phrase was entered.  We give the
-             * user (but not the dialog program) a few more
-             * chances...
+             * check algorithm type of private key and make
+             * sure only one private key per type is used.
              */
-            if (   sc->nPassPhraseDialogType == SSL_PPTYPE_BUILTIN
-                && cpPassPhraseCur != NULL
-                && nPassPhraseRetry < BUILTIN_DIALOG_RETRIES ) {
-                fprintf(stdout, "Apache:mod_ssl:Error: Pass phrase incorrect "
-                        "(%d more retr%s permitted).\n",
-                        (BUILTIN_DIALOG_RETRIES-nPassPhraseRetry),
-                        (BUILTIN_DIALOG_RETRIES-nPassPhraseRetry) == 1 ? "y" : "ies");
-                nPassPhraseRetry++;
-                if (nPassPhraseRetry > BUILTIN_DIALOG_BACKOFF)
-                    sleep((nPassPhraseRetry-BUILTIN_DIALOG_BACKOFF)*5);
-                continue;
+            at = ssl_util_algotypeof(NULL, pPrivateKey);
+            an = ssl_util_algotypestr(at);
+            if (algoKey & at) {
+                ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                        "Init: Multiple %s server private keys not allowed", an);
+                ssl_die();
+            }
+            algoKey |= at;
+
+            /*
+             * Log the type of reading
+             */
+            if (nPassPhraseDialogCur == 0)
+                ssl_log(pServ, SSL_LOG_TRACE, 
+                        "Init: (%s) unencrypted %s private key - pass phrase not required", 
+                        cpVHostID, an);
+            else {
+                if (cpPassPhraseCur != NULL)
+                    ssl_log(pServ, SSL_LOG_TRACE, 
+                            "Init: (%s) encrypted %s private key - pass phrase requested", 
+                            cpVHostID, an);
+                else
+                    ssl_log(pServ, SSL_LOG_TRACE, 
+                            "Init: (%s) encrypted %s private key - pass phrase reused", 
+                            cpVHostID, an);
             }
 
             /*
-             * Ok, anything else now means a fatal error.
+             * Ok, when we have one more pass phrase store it
              */
-            if (sc->nPassPhraseDialogType == SSL_PPTYPE_BUILTIN) {
-                fprintf(stdout, "Apache:mod_ssl:Error: Pass phrase incorrect.\n");
-                fprintf(stdout, "**Stopped\n");
+            if (cpPassPhraseCur != NULL) {
+                cpp = (char **)ssl_ds_array_push(aPassPhrase);
+                *cpp = cpPassPhraseCur;
+                nPassPhrase++;
             }
-            ssl_log(pServ, SSL_LOG_ERROR, "Init: Pass phrase incorrect.");
-            ssl_die();
+
+            /*
+             * Insert private key into the global module configuration
+             * (we convert it to a stand-alone DER byte sequence
+             * because the SSL library uses static variables inside a
+             * RSA structure which do not survive DSO reloads!)
+             */
+            cp = ap_psprintf(mc->pPool, "%s:%s", cpVHostID, an);
+            asn1 = (ssl_asn1_t *)ssl_ds_table_push(mc->tPrivateKey, cp);
+            asn1->nData  = i2d_PrivateKey(pPrivateKey, NULL);
+            asn1->cpData = ap_palloc(mc->pPool, asn1->nData);
+            ucp = asn1->cpData; i2d_PrivateKey(pPrivateKey, &ucp); /* 2nd arg increments */
+
+            /*
+             * Free the private key structure
+             */
+            EVP_PKEY_free(pPrivateKey);
         }
-
-        /*
-         * Log the type of reading
-         */
-        if (nPassPhraseDialogCur == 0)
-            ssl_log(pServ, SSL_LOG_TRACE,
-                    "Init: (%s) unencrypted private key - pass phrase not required",
-                    cpVHostID);
-        else {
-            if (cpPassPhraseCur != NULL)
-                ssl_log(pServ, SSL_LOG_TRACE,
-                        "Init: (%s) encrypted private key - pass phrase requested",
-                        cpVHostID);
-            else
-                ssl_log(pServ, SSL_LOG_TRACE,
-                        "Init: (%s) encrypted private key - pass phrase reused",
-                        cpVHostID);
-        }
-
-        /*
-         * Ok, when have one more pass phrase store it
-         */
-        if (cpPassPhraseCur != NULL) {
-            cpp = (char **)ssl_ds_array_push(aPassPhrase);
-            *cpp = cpPassPhraseCur;
-            nPassPhrase++;
-        }
-
-        /*
-         * Insert private key into the global module configuration
-         * (we convert it to a stand-alone DER byte sequence
-         * because the SSL library uses static variables inside a
-         * RSA structure which do not survive DSO reloads!)
-         */
-        asn1 = (ssl_asn1_t *)ssl_ds_table_push(mc->tPrivateKey, cpVHostID);
-        asn1->nData  = i2d_RSAPrivateKey(pRSAKey, NULL);
-        asn1->cpData = ap_palloc(mc->pPool, asn1->nData);
-        ucp = asn1->cpData; i2d_RSAPrivateKey(pRSAKey, &ucp); /* 2nd arg increments */
-
-        /*
-         * Free the RSA structure
-         */
-        RSA_free(pRSAKey);
     }
 
     /*
