@@ -1,8 +1,8 @@
 /*                      _             _
-**  _ __ ___   ___   __| |    ___ ___| |
-** | '_ ` _ \ / _ \ / _` |   / __/ __| |
-** | | | | | | (_) | (_| |   \__ \__ \ | mod_ssl - Apache Interface to OpenSSL
-** |_| |_| |_|\___/ \__,_|___|___/___/_| http://www.engelschall.com/sw/mod_ssl/
+**  _ __ ___   ___   __| |    ___ ___| |  mod_ssl
+** | '_ ` _ \ / _ \ / _` |   / __/ __| |  Apache Interface to OpenSSL
+** | | | | | | (_) | (_| |   \__ \__ \ |  www.modssl.org
+** |_| |_| |_|\___/ \__,_|___|___/___/_|  ftp.modssl.org
 **                      |_____|
 **  ssl_engine_vars.c
 **  Variable Lookup Facility
@@ -27,7 +27,7 @@
  *    software must display the following acknowledgment:
  *    "This product includes software developed by
  *     Ralf S. Engelschall <rse@engelschall.com> for use in the
- *     mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)."
+ *     mod_ssl project (http://www.modssl.org/)."
  *
  * 4. The names "mod_ssl" must not be used to endorse or promote
  *    products derived from this software without prior written
@@ -42,7 +42,7 @@
  *    acknowledgment:
  *    "This product includes software developed by
  *     Ralf S. Engelschall <rse@engelschall.com> for use in the
- *     mod_ssl project (http://www.engelschall.com/sw/mod_ssl/)."
+ *     mod_ssl project (http://www.modssl.org/)."
  *
  * THIS SOFTWARE IS PROVIDED BY RALF S. ENGELSCHALL ``AS IS'' AND ANY
  * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -77,7 +77,7 @@ static char *ssl_var_lookup_ssl_cert(pool *p, X509 *xs, char *var);
 static char *ssl_var_lookup_ssl_cert_dn(pool *p, X509_NAME *xsname, char *var);
 static char *ssl_var_lookup_ssl_cert_valid(pool *p, ASN1_UTCTIME *tm);
 static char *ssl_var_lookup_ssl_cert_serial(pool *p, X509 *xs);
-static char *ssl_var_lookup_ssl_cert_chain(pool *p, STACK *sk, char *var);
+static char *ssl_var_lookup_ssl_cert_chain(pool *p, STACK_OF(X509) *sk, char *var);
 static char *ssl_var_lookup_ssl_cert_PEM(pool *p, X509 *xs);
 static char *ssl_var_lookup_ssl_cipher(pool *p, conn_rec *c, char *var);
 static void  ssl_var_lookup_ssl_cipher_bits(char *cipher, int *usekeysize, int *algkeysize);
@@ -279,33 +279,35 @@ static char *ssl_var_lookup_ssl(pool *p, conn_rec *c, char *var)
 {
     char *result;
     X509 *xs;
-    STACK *sk;
+    STACK_OF(X509) *sk;
     SSL *ssl;
 
     result = NULL;
 
+    ssl = ap_ctx_get(c->client->ctx, "ssl");
     if (strlen(var) > 8 && strcEQn(var, "VERSION_", 8)) {
         result = ssl_var_lookup_ssl_version(p, var+8);
     }
-    else if (strcEQ(var, "PROTOCOL")) {
-        ssl = ap_ctx_get(c->client->ctx, "ssl");
+    else if (ssl != NULL && strcEQ(var, "PROTOCOL")) {
         result = SSL_get_version(ssl);
     }
-    else if (strlen(var) >= 6 && strcEQn(var, "CIPHER", 6)) {
+    else if (ssl != NULL && strcEQ(var, "SESSION_ID")) {
+        SSL_SESSION *pSession = SSL_get_session(ssl);
+        result = ap_pstrdup(p, ssl_scache_id2sz(pSession->session_id, 
+                                                pSession->session_id_length));
+    }
+    else if (ssl != NULL && strlen(var) >= 6 && strcEQn(var, "CIPHER", 6)) {
         result = ssl_var_lookup_ssl_cipher(p, c, var+6);
     }
-    else if (strlen(var) > 18 && strcEQn(var, "CLIENT_CERT_CHAIN_", 18)) {
-        ssl = ap_ctx_get(c->client->ctx, "ssl");
+    else if (ssl != NULL && strlen(var) > 18 && strcEQn(var, "CLIENT_CERT_CHAIN_", 18)) {
         sk = SSL_get_peer_cert_chain(ssl);
         result = ssl_var_lookup_ssl_cert_chain(p, sk, var+17);
     }
-    else if (strlen(var) > 7 && strcEQn(var, "CLIENT_", 7)) {
-        ssl = ap_ctx_get(c->client->ctx, "ssl");
+    else if (ssl != NULL && strlen(var) > 7 && strcEQn(var, "CLIENT_", 7)) {
         if ((xs = SSL_get_peer_certificate(ssl)) != NULL)
             result = ssl_var_lookup_ssl_cert(p, xs, var+7);
     }
-    else if (strlen(var) > 7 && strcEQn(var, "SERVER_", 7)) {
-        ssl = ap_ctx_get(c->client->ctx, "ssl");
+    else if (ssl != NULL && strlen(var) > 7 && strcEQn(var, "SERVER_", 7)) {
         if ((xs = SSL_get_certificate(ssl)) != NULL)
             result = ssl_var_lookup_ssl_cert(p, xs, var+7);
     }
@@ -403,12 +405,15 @@ static char *ssl_var_lookup_ssl_cert_dn(pool *p, X509_NAME *xsname, char *var)
 
     for (i = 0; ssl_var_lookup_ssl_cert_dn_rec[i].name != NULL; i++) {
         if (strEQ(var, ssl_var_lookup_ssl_cert_dn_rec[i].name)) {
-            for (j = 0; j < sk_num(xsname->entries); j++) {
-                xsne = (X509_NAME_ENTRY *)sk_value(xsname->entries, j);
+            for (j = 0; j < sk_X509_NAME_ENTRY_num(xsname->entries); j++) {
+                xsne = sk_X509_NAME_ENTRY_value(xsname->entries, j);
                 n = OBJ_obj2nid(xsne->object);
                 if (n == ssl_var_lookup_ssl_cert_dn_rec[i].nid) {
                     result = ap_palloc(p, xsne->value->length+1);
                     ap_cpystrn(result, (char *)xsne->value->data, xsne->value->length+1);
+#ifdef CHARSET_EBCDIC
+                    ascii2ebcdic(result, result, xsne->value->length);
+#endif /* CHARSET_EBCDIC */
                     result[xsne->value->length] = NUL;
                     break;
                 }
@@ -453,7 +458,7 @@ static char *ssl_var_lookup_ssl_cert_serial(pool *p, X509 *xs)
     return result;
 }
 
-static char *ssl_var_lookup_ssl_cert_chain(pool *p, STACK *sk, char *var)
+static char *ssl_var_lookup_ssl_cert_chain(pool *p, STACK_OF(X509) *sk, char *var)
 {
     char *result;
     X509 *xs;
@@ -463,8 +468,8 @@ static char *ssl_var_lookup_ssl_cert_chain(pool *p, STACK *sk, char *var)
 
     if (strspn(var, "0123456789") == strlen(var)) {
         n = atoi(var);
-        if (sk_num(sk) >= n) {
-            xs = (X509 *)sk_value(sk, n);
+        if (sk_X509_num(sk) >= n) {
+            xs = sk_X509_value(sk, n);
             result = ssl_var_lookup_ssl_cert_PEM(p, xs);
         }
     }
@@ -502,24 +507,24 @@ static char *ssl_var_lookup_ssl_cipher(pool *p, conn_rec *c, char *var)
 
     if (strEQ(var, "")) {
         ssl = ap_ctx_get(c->client->ctx, "ssl");
-        result = SSL_get_cipher_name(ssl);
+        result = (char *)SSL_get_cipher_name(ssl);
     }
     else if (strcEQ(var, "_EXPORT")) {
         ssl = ap_ctx_get(c->client->ctx, "ssl");
-        cipher = SSL_get_cipher_name(ssl);
+        cipher = (char *)SSL_get_cipher_name(ssl);
         ssl_var_lookup_ssl_cipher_bits(cipher, &usekeysize, &algkeysize);
         result = (usekeysize < 56 ? "true" : "false");
     }
     else if (strcEQ(var, "_USEKEYSIZE")) {
         ssl = ap_ctx_get(c->client->ctx, "ssl");
-        cipher = SSL_get_cipher_name(ssl);
+        cipher = (char *)SSL_get_cipher_name(ssl);
         ssl_var_lookup_ssl_cipher_bits(cipher, &usekeysize, &algkeysize);
         result = ap_psprintf(p, "%d", usekeysize);
         resdup = FALSE;
     }
     else if (strcEQ(var, "_ALGKEYSIZE")) {
         ssl = ap_ctx_get(c->client->ctx, "ssl");
-        cipher = SSL_get_cipher_name(ssl);
+        cipher = (char *)SSL_get_cipher_name(ssl);
         ssl_var_lookup_ssl_cipher_bits(cipher, &usekeysize, &algkeysize);
         result = ap_psprintf(p, "%d", algkeysize);
         resdup = FALSE;
@@ -540,11 +545,11 @@ static const struct {
     int nUseKeySize;
     int nAlgKeySize;
 } ssl_var_lookup_ssl_cipher_bits_rec[] = {
-#if SSL_LIBRARY_VERSION >= 0x0920
-    { TLS1_TXT_RSA_EXPORT56_WITH_RC4_56_MD5     /*EXP56-RC4-MD5*/,    56, 128 },
-    { TLS1_TXT_RSA_EXPORT56_WITH_RC2_CBC_56_MD5 /*EXP56-RC2-CBC-MD5*/,56, 128 },
-    { TLS1_TXT_RSA_EXPORT56_WITH_DES_CBC_SHA    /*EXP56-DES-CBC-SHA*/,56,  56 },
-#endif
+
+    { TLS1_TXT_RSA_EXPORT1024_WITH_RC4_56_MD5     /*EXP1024-RC4-MD5*/,    56, 128 },
+    { TLS1_TXT_RSA_EXPORT1024_WITH_RC2_CBC_56_MD5 /*EXP1024-RC2-CBC-MD5*/,56, 128 },
+    { TLS1_TXT_RSA_EXPORT1024_WITH_DES_CBC_SHA    /*EXP1024-DES-CBC-SHA*/,56,  56 },
+
     { SSL3_TXT_RSA_IDEA_128_SHA          /*IDEA-CBC-SHA*/,           128, 128 },
     { SSL3_TXT_RSA_NULL_MD5              /*NULL-MD5*/,                 0,   0 },
     { SSL3_TXT_RSA_NULL_SHA              /*NULL-SHA*/,                 0,   0 },
@@ -576,12 +581,14 @@ static const struct {
     { SSL3_TXT_FZA_DMS_NULL_SHA          /*FZA-NULL-SHA*/,             0,   0 },
     { SSL3_TXT_FZA_DMS_FZA_SHA           /*FZA-FZA-CBC-SHA*/,          0,   0 },
     { SSL3_TXT_FZA_DMS_RC4_SHA           /*FZA-RC4-SHA*/,            128, 128 },
+
     { SSL2_TXT_DES_64_CFB64_WITH_MD5_1   /*DES-CFB-M1*/,              56,  56 },
     { SSL2_TXT_RC2_128_CBC_WITH_MD5      /*RC2-CBC-MD5*/,            128, 128 },
     { SSL2_TXT_DES_64_CBC_WITH_MD5       /*DES-CBC-MD5*/,             56,  56 },
     { SSL2_TXT_DES_192_EDE3_CBC_WITH_MD5 /*DES-CBC3-MD5*/,           168, 168 },
     { SSL2_TXT_RC4_64_WITH_MD5           /*RC4-64-MD5*/,              64,  64 },
     { SSL2_TXT_NULL                      /*NULL*/,                     0,   0 },
+
     { NULL,                                                            0,   0 }
 };
 
@@ -619,7 +626,7 @@ static char *ssl_var_lookup_ssl_version(pool *p, char *var)
         result = ap_psprintf(p, "mod_ssl/%s", MOD_SSL_VERSION);
     }
     else if (strEQ(var, "LIBRARY")) {
-        result = ap_pstrdup(p, OpenSSL_version(OPENSSL_VERSION));
+        result = ap_pstrdup(p, SSL_LIBRARY_TEXT);
         if ((cp = strchr(result, ' ')) != NULL) {
             *cp = '/';
             if ((cp2 = strchr(cp, ' ')) != NULL)
