@@ -112,12 +112,13 @@ X509 *SSL_read_X509(FILE *fp, X509 **x509, int (*cb)())
         if ((bioS = BIO_new(BIO_s_fd())) == NULL)
             return NULL;
         BIO_set_fd(bioS, fileno(fp), BIO_NOCLOSE);
-        if ((bioF = BIO_new(BIO_f_base64())) == NULL)
-             return NULL;
+        if ((bioF = BIO_new(BIO_f_base64())) == NULL) {
+            BIO_free(bioS);
+            return NULL;
+        }
         bioS = BIO_push(bioF, bioS);
         rc = d2i_X509_bio(bioS, NULL);
-        BIO_free(bioF);
-        BIO_free(bioS);
+        BIO_free_all(bioS);
         if (rc == NULL) {
             /* 3. try plain DER */
             fseek(fp, 0L, SEEK_SET);
@@ -158,12 +159,13 @@ EVP_PKEY *SSL_read_PrivateKey(FILE *fp, EVP_PKEY **key, int (*cb)())
         if ((bioS = BIO_new(BIO_s_fd())) == NULL)
             return NULL;
         BIO_set_fd(bioS, fileno(fp), BIO_NOCLOSE);
-        if ((bioF = BIO_new(BIO_f_base64())) == NULL)
-             return NULL;
+        if ((bioF = BIO_new(BIO_f_base64())) == NULL) {
+            BIO_free(bioS);
+            return NULL;
+        }
         bioS = BIO_push(bioF, bioS);
         rc = d2i_PrivateKey_bio(bioS, NULL);
-        BIO_free(bioF);
-        BIO_free(bioS);
+        BIO_free_all(bioS);
         if (rc == NULL) {
             /* 3. try plain DER */
             fseek(fp, 0L, SEEK_SET);
